@@ -1,8 +1,8 @@
 #
 # -*- Perl -*-
-# $Id: html.pl,v 1.32.4.9 2004/03/20 15:48:30 usu Exp $
+# $Id: html.pl,v 1.32.4.16 2005/12/06 19:52:57 opengl2772 Exp $
 # Copyright (C) 1997-1999 Satoru Takabayashi All rights reserved.
-# Copyright (C) 2000-2003 Namazu Project All rights reserved.
+# Copyright (C) 2000-2005 Namazu Project All rights reserved.
 #     This is free software with ABSOLUTELY NO WARRANTY.
 #
 #  This program is free software; you can redistribute it and/or modify
@@ -25,6 +25,7 @@
 
 package html;
 use strict;
+require 'util.pl';
 require 'gfilter.pl';
 
 my $EMBEDDED_FILE = '\.(asp|jsp|php[3s]?|phtml)(?:\.gz)?';
@@ -101,9 +102,9 @@ sub html_filter ($$$$) {
     html::get_author($contref, $fields);
     html::get_meta_tags($contref, $weighted_str, $fields);
 #    html::get_img_alt($contref);
-    html::get_alt_attr($contref) if "ALT" =~ /^($conf::HTML_ATTRIBUTES)$/io;
-    html::get_table_summary($contref) if "SUMMARY" =~ /^($conf::HTML_ATTRIBUTES)$/io;
-    html::get_title_attr($contref) if "TITLE" =~ /^($conf::HTML_ATTRIBUTES)$/io;
+    html::get_alt_attr($contref) if "ALT" =~ /^(?:$conf::HTML_ATTRIBUTES)$/io;
+    html::get_table_summary($contref) if "SUMMARY" =~ /^(?:$conf::HTML_ATTRIBUTES)$/io;
+    html::get_title_attr($contref) if "TITLE" =~ /^(?:$conf::HTML_ATTRIBUTES)$/io;
     html::normalize_html_element($contref);
     html::erase_above_body($contref);
     html::remove_comments($contref);
@@ -144,11 +145,14 @@ sub escape_lt_gt ($) {
 sub get_author ($$) {
     my ($contref, $fields) = @_;
 
+    # <META NAME="AUTHOR" CONTENT="author">
     # <LINK REV=MADE HREF="mailto:ccsatoru@vega.aichi-u.ac.jp">
 
-    if ($$contref =~ m!<LINK\s[^>]*?HREF=([\"\'])mailto:(.*?)\1\s*>!i) { #"
-	    $fields->{'author'} = $2;
-    } elsif ($$contref =~ m!.*<ADDRESS[^>]*>(.*?)</ADDRESS>!is) {
+    if ($$contref =~ m!<META\s[^>]*?NAME=([\"\']?)AUTHOR\1\s[^>]*?CONTENT=([\"\']?)(.*?)\2\s*>!is) {
+        $fields->{'author'} = $3;
+    } elsif ($$contref =~ m!<LINK\s[^>]*?HREF=([\"\']?)mailto:(.*?)\1\s*>!i) {
+        $fields->{'author'} = $2;
+    } elsif ($$contref =~ m!<ADDRESS[^>]*>(.*?)</ADDRESS>!is) {
 	my $tmp = $1;
 #	$tmp =~ s/\s//g;
 	if ($tmp =~ /\b([\w\.\-]+\@[\w\.\-]+(?:\.[\w\.\-]+)+)\b/) {
@@ -196,16 +200,19 @@ sub get_meta_tags ($$$) {
 	    \1\s+[^>]*content\s*=\s*([\'\"]?)([^>]*?)\2[^>]*>/ix; #"
 
     if ($var::Opt{'meta'}) {
-	my @keys = split '\|', $conf::META_TAGS;
-	for my $key (@keys) {
-	    while ($$contref =~ /<meta\s+name\s*=\s*([\'\"]?)$key #"
-	       \1\s+[^>]*content\s*=\s*([\'\"]?)([^>]*?)\2[^>]*>/gix) 
-	    {
-		$fields->{$key} .= $3 . " ";
-	    }
-	    util::dprint("meta: $key: $fields->{$key}\n") 
-		if defined $fields->{$key};
-	}
+        my @keys = split '\|', $conf::META_TAGS;
+        for my $key (@keys) {
+            if ($key !~ m/^author$/i) {
+                my $quotekey = quotemeta($key);
+                while ($$contref =~ /<meta\s+name\s*=\s*([\'\"]?)$quotekey #"
+                    \1\s+[^>]*content\s*=\s*([\'\"]?)([^>]*?)\2[^>]*>/gix)
+                {
+                    $fields->{$key} .= $3 . " ";
+                }
+                util::dprint("meta: $key: $fields->{$key}\n")
+                    if defined $fields->{$key};
+            }
+        }
     }
 }
 
@@ -306,6 +313,9 @@ sub element_space ($) {
 sub remove_html_elements ($) {
     my ($contref) = @_;
 
+    # remove Office Markup <o:></o:>, <![]>
+    $$contref =~ s#</?([A-Z]\w*):.*?>|<(!)\[.*?\]\s*>#element_space($1||$2)#gsixe;
+
     # remove all elements
     $$contref =~ s!</?([A-Z]\w*)(?:\s+[A-Z]\w*(?:\s*=\s*(?:(["']).*?\2|[\w\-.]+))?)*\s*>!element_space($1)!gsixe;
 
@@ -314,8 +324,11 @@ sub remove_html_elements ($) {
 # Decode a numberd entity. Exclude an invalid number.
 sub decode_numbered_entity ($) {
     my ($num) = @_;
+
+    # FIXME: very ad hoc. (ISO-8859-1)
     return ""
-	if $num >= 0 && $num <= 31 || $num >= 127 && $num <= 159 ;
+	if (($num >= 0 && $num <= 31) || ($num >= 127 && $num <= 159) ||
+        ($num >= 256));
     return "" 
 	if $num >=127 && util::islang('ja');
     sprintf ("%c",$num);
