@@ -1,9 +1,9 @@
 #
 # -*- Perl -*-
-# $Id: mailnews.pl,v 1.27.4.2 2004/03/20 15:48:30 usu Exp $
+# $Id: mailnews.pl,v 1.27.4.12 2006/01/29 13:26:44 opengl2772 Exp $
 # Copyright (C) 1997-2000 Satoru Takabayashi ,
 #               1999 NOKUBI Takatsugu ,
-#               2001,2003 Namazu Project All rights reserved.
+#               2001,2003-2006 Namazu Project All rights reserved.
 #     This is free software with ABSOLUTELY NO WARRANTY.
 #
 #  This program is free software; you can redistribute it and/or modify
@@ -28,12 +28,18 @@ package mailnews;
 use strict;
 require 'util.pl';
 require 'gfilter.pl';
+require 'html.pl';
+
+my $has_base64 = undef;
+my $htmlmail = "";
 
 sub mediatype() {
     return ('message/rfc822', 'message/news');
 }
 
 sub status() {
+    $has_base64 = 1 if (util::checklib('MIME/Base64.pm') &&
+			util::checklib('MIME/QuotedPrint.pm'));
     return 'yes';
 }
 
@@ -62,6 +68,9 @@ sub filter ($$$$$) {
 
     uuencode_filter($cont);
     mailnews_filter($cont, $weighted_str, $fields);
+    if ($htmlmail) {
+       html::html_filter($cont, $weighted_str, $fields, $headings);
+    }
     mailnews_citation_filter($cont, $weighted_str);
 
     gfilter::line_adjust_filter($cont);
@@ -81,6 +90,8 @@ sub mailnews_filter ($$$) {
     my $line     = "";
     my $partial  = 0;
 
+    $htmlmail = "";
+
     $$contref =~ s/^\s+//;
     # Don't handle if first like does'nt seem like a mail/news header.
     return unless $$contref =~ /(^\S+:|^from )/i;
@@ -112,15 +123,20 @@ sub mailnews_filter ($$$) {
 
 	    my $weight = $conf::Weight{'html'}->{'title'};
 	    $$weighted_str .= "\x7f$weight\x7f$line\x7f/$weight\x7f\n";
- 	} elsif ($line =~ s/^content-type:\s*//i) {
-	    if ($line =~ /multipart.*boundary="(.*)"/i){
-		$boundary = $1;
-		util::dprint("((boundary: $boundary))\n");
-  	    } elsif ($line =~ m!message/partial;\s*(.*)!i) {
-		# The Message/Partial subtype routine [1998-10-12]
-		# contributed by Hiroshi Kato <tumibito@mm.rd.nttdata.co.jp>
-  		$partial = $1;
-  		util::dprint("((partial: $partial))\n");
+        } elsif ($line =~ s/^content-type:\s*//i) {
+            if ($line =~ /multipart.*boundary="(.*?)"/si ||
+            $line =~ /multipart.*boundary=(.*?)(?:\s|$)/mi) {
+                $boundary = $1;
+                util::dprint("((boundary: $boundary))\n");
+            } elsif ($line =~ m!message/partial;\s*(.*)!i) {
+                # The Message/Partial subtype routine [1998-10-12]
+                # contributed by Hiroshi Kato <tumibito@mm.rd.nttdata.co.jp>
+                $partial = $1;
+                util::dprint("((partial: $partial))\n");
+            } elsif ($line =~ m!text/html!i) {
+               # The simplest form of an HTML email message.
+               util::dprint("text/html mail\n");
+               $htmlmail = "yes";
             } elsif ($line !~ m!text/plain!i) {
                 $$contref = '';
                 return;
@@ -154,21 +170,69 @@ sub mailnews_filter ($$$) {
 	$boundary =~ s/(\W)/\\$1/g;
 	$$contref =~ s/This is multipart message.\n//i;
 
+	multipart_process($contref, $boundary, $weighted_str, $fields);
 
-	# MIME multipart processing,
-	# modified by Furukawa-san's patch on [1998/08/27]
- 	$$contref =~ s/--$boundary(--)?\n?/\xff/g;
- 	my (@parts) = split(/\xff/, $$contref);
- 	$$contref = '';
- 	for $_ (@parts){
- 	    if (s/^(.*?\n\n)//s){
- 		my ($head) = $1;
- 		$$contref .= $_ if $head =~ m!^content-type:.*text/plain!mi;
- 	    }
- 	}
     }
 }
 
+# Prototype declaration for avoiding
+# "multipart_process() called too early to check prototype at ..." warnings.
+sub multipart_process($$$$);
+
+sub multipart_process ($$$$){
+    my ($contref, $boundary, $weighted_str, $fields) = @_;
+
+    # MIME multipart processing,
+    # modified by Furukawa-san's patch on [1998/08/27]
+    $$contref =~ s/--$boundary(--)?\n?/\xff/g;
+    my (@parts) = split(/\xff/, $$contref);
+    $$contref = '';
+    for $_ (@parts){
+	if (s/^(.*?\n\n)//s){
+	    my ($head) = $1; 
+	    my ($body) .= $_;
+	    my $contenttype = "";
+	    my $cont_encode = "";
+	    if ($head =~ m!^content-type:\s*(\S+?);?\s!mi){
+		$contenttype = lc($1);
+		util::dprint("((Content-Type: $contenttype))\n");
+	    }
+
+	    if ($head =~ m!^content-transfer-encoding:\s*(\S+)$!mi){
+		$cont_encode = lc($1);
+		util::dprint("((Content-Transfer-Encode: $cont_encode))\n");
+	    }
+
+	    # Image data must not be including text data.  
+	    if ($contenttype !~ m!image/!){
+		if ($cont_encode =~ m/base64/){
+		    base64_filter(\$body);
+		} elsif ($cont_encode =~ m/quoted-printable/){
+		    quotedprint_filter(\$body);
+		} 
+
+		if ($contenttype =~ m!text/plain!){
+		    $$contref .= $body;
+		} elsif ($contenttype =~ m!multipart/alternative!){
+                    if ($head =~ /boundary="(.*?)"/si ||
+                    $head =~ /boundary=(.*?)(?:\s|$)/mi) {
+			my $boundary2 = $1;
+			util::dprint("((boundary: $boundary2))\n");
+			$boundary2 =~ s/(\W)/\\$1/g;
+			multipart_process(\$body, $boundary2, $weighted_str, $fields);
+			$$contref .= $body;
+		    }
+		} elsif ($body ne "") {
+		    nesting_filter(\$head, \$body, $contenttype, $weighted_str);
+		    $$contref .= $body;
+		}
+	    }
+	}
+        $$contref .= " ";
+    }
+}
+
+
 # Make mail/news citation marks not to be indexed.
 # And a greeting message at the beginning.
 # And a meaningless message such as "foo wrote:".
@@ -213,7 +277,7 @@ sub mailnews_citation_filter ($$) {
         # Process only first five paragrahs.
 	# And don't handle the paragrah which has five or longer lines.
 	# Hmm, this regex looks very hairly.
-	if ($i < 5 && ($line =~ tr/\n/\n/) <= 5 && $line =~ /(^\s*(Date:|Subject:|Message-ID:|From:|̾|п|))|(^.+(ֻǤ|reply\s*Ǥ|۩|勞|񤭤ޤ|ޤ|ä|wrote|said|writes|says)(.{0,2})?\s*$)|(^.*In .*(article|message))|(<\S+\@([\w\-.]\.)+\w+>)/im) {
+	if ($i < 5 && ($line =~ tr/\n/\n/) <= 5 && $line =~ /(^\s*(Date:|Subject:|Message-ID:|From:|̾|п|))|(^.+(ֻǤ|reply\s*Ǥ|۩|勞|񤭤ޤ|ޤ|ä|wrote|said|writes|says)(.{0,2})?\s*$)|(^.*In .*(article|message))|<\S+\@[\w\-\.]+?\.\w+>/im) {
 	    util::dprint("\n\n<<<<$line>>>>\n\n");
 	    $omake .= $line . "\n";
 	    $line = "";
@@ -298,5 +362,89 @@ sub uuencode_filter ($) {
     }
 }
 
+sub base64_filter ($){
+    my ($bodyref) = @_;
+    if ($has_base64 && $var::Opt{'decodebase64'}) {
+	eval 'use MIME::Base64 ();';
+	$$bodyref = MIME::Base64::decode($$bodyref);
+    } else {
+	$$bodyref="";
+    }
+}
+
+sub quotedprint_filter ($){
+    my ($bodyref) = @_;
+    if ($has_base64 && $var::Opt{'decodebase64'}) {
+	eval 'use MIME::QuotedPrint ();';
+	$$bodyref = MIME::QuotedPrint::decode_qp($$bodyref);
+    } else {
+	$$bodyref="";
+    }
+}
+
+sub nesting_filter ($$$$){
+    my ($headref, $bodyref, $mmtype, $weighted_str) = @_;
+    my $err = undef;
+    my $dummy_shelterfname="";
+    my $headings = "";
+    my %fields;
+    my $filename = "";
+    if ($$headref =~ m!^content-disposition:\s*\S+\s*filename="(.+?)"!smi){
+	$filename = $1;
+
+	#AL-Mail divides filename into some lines when MIME B encoding.
+	$filename =~ s/\s+//g;
+
+    } elsif ($$headref =~ m!^content-location:\s*(\S+)!mi){
+	$filename = $1;
+
+    } elsif ($$headref =~ m!^content-disposition:\s*\S+\s*(filename\*.+?[^;])$!smi){
+	#RFC2231 MIME encoded
+	$filename =$1;
+	$filename =~ s/;.+?=//smg;
+	(my $charset, my $lang, my $tmp) = ($filename =~ /=(.+)'(.+)'(.+)/);
+	if ($tmp) {
+	    $tmp =~ s/%(\w\w)/chr(hex($1))/eg;
+	    $filename = $tmp;
+	    # codeconv::toeuc(\$filename);
+            codeconv::codeconv_document(\$filename);
+	}else {
+	    $filename =~ s/filename.*=//;
+	}
+    }
+    util::dprint("((Attached filename: $filename))\n");
+
+    if ($filename =~ m!^($conf::DENY_FILE)$!i ) {
+	util::vprint(sprintf(_("Denied:	%s"), $filename));
+	$err = "Denied file.";
+	return $err;
+    } elsif ($filename !~ m!^($conf::ALLOW_FILE)$!i) {
+	util::vprint(sprintf(_("Not allowed:	%s"), $filename));
+	$err = "Not allowed file.";
+	return $err;
+    }
+
+    #if ($mmtype =~ m!application/octet-stream!){
+	$mmtype = undef;
+    #}
+
+    my ($kanji, $mtype) = mknmz::apply_filter(\$filename, $bodyref, 
+			$weighted_str, \$headings, \%fields, 
+			$dummy_shelterfname, $mmtype);
+    if ($mtype =~ /; x-system=unsupported$/){
+	$$bodyref = "";
+        $err = $mtype;
+	util::dprint("filter/mailnews.pl gets error message \"$err\"");
+    }elsif ($mtype =~ /; x-error=(.*)$/){
+        $$bodyref = "";
+        $err = $1;
+        util::dprint("filter/mailnews.pl gets error message \"$err\"");
+    }else{
+	$$bodyref .= " ". $filename;
+	gfilter::show_filter_debug_info($bodyref, $weighted_str,
+					\%fields, \$headings);
+    }
+    return $err;
+}
 
 1;
