@@ -1,9 +1,9 @@
 /*
  * 
- * $Id: search.c,v 1.86.4.12 2004/04/04 15:26:48 opengl2772 Exp $
+ * $Id: search.c,v 1.86.4.20 2005/12/27 16:10:45 opengl2772 Exp $
  * 
  * Copyright (C) 1997-1999 Satoru Takabayashi All rights reserved.
- * Copyright (C) 2000-2004 Namazu Project All rights reserved.
+ * Copyright (C) 2000-2005 Namazu Project All rights reserved.
  * This is free software with ABSOLUTELY NO WARRANTY.
  * 
  * This program is free software; you can redistribute it and/or modify
@@ -100,9 +100,11 @@ static void close_index_files ( void );
 static void do_logging ( const char * query, int n );
 static NmzResult nmz_search_sub ( NmzResult hlist, const char *query, int n );
 static void make_fullpathname_index ( int n );
-static void remove_quotes(char *str);
-static enum nmz_stat normalize_idxnames(void);
+static void remove_quotes( char *str );
+static enum nmz_stat normalize_idxnames( void );
 static int issymbol ( int c );
+static void escape_meta_characters( char *expr, size_t bufsize );
+static void nmz_regex_strlower( char *str );
 
 /*
  * Show the status for debug use
@@ -158,7 +160,7 @@ lrget(int *l, int *r)
 static NmzResult 
 prefix_match(const char *key, int v)
 {
-    int i, j, n;
+    int i, j, n, maxmatch, maxhit;
     char buf[BUFSIZE], tmpkey[BUFSIZE];
     NmzResult val, tmp;
 
@@ -184,38 +186,47 @@ prefix_match(const char *key, int v)
 	v = i;
     }
 
-    for (j = 0, i++;; i++, j++) {
+    maxmatch = nmz_get_maxmatch();
+    maxhit = nmz_get_maxhit();
+
+    for (j = 0, i++;; i++) {
 	/* 
 	 * Return if too much word would be hit
          * because treat 'a*' completely is too consuming 
 	 */
-	if (j > nmz_get_maxmatch()) {
-	    nmz_free_hlist(val);
-	    val.stat = ERR_TOO_MUCH_MATCH;
+	if (fseek(Nmz.w, nmz_getidxptr(Nmz.wi, i), 0) != 0) {
 	    break;
 	}
-	if (-1 == fseek(Nmz.w, nmz_getidxptr(Nmz.wi, i), 0)) {
-	    break;
-	}
-	fgets(buf, BUFSIZE - 1, Nmz.w);
+        if (fgets(buf, BUFSIZE - 1, Nmz.w) == NULL) {
+            break;
+        }
         nmz_chomp(buf);
 	if (strncmp(tmpkey, buf, n) == 0) {
 	    tmp = nmz_get_hlist(i);
-	    if (tmp.stat == ERR_FATAL)
-	        return tmp;
-	    if (tmp.num > nmz_get_maxhit()) {
+            if (tmp.stat == ERR_FATAL) {
+                return tmp;
+            }
+	    if (tmp.num > maxhit) {
 		nmz_free_hlist(val);
-		val.stat = ERR_TOO_MUCH_MATCH;
-		break;
-	    }
-	    val = nmz_ormerge(val, tmp);
-	    if (val.stat == ERR_FATAL)
-	        return val;
-	    if (val.num > nmz_get_maxhit()) {
-		nmz_free_hlist(val);
-		val.stat = ERR_TOO_MUCH_MATCH;
+		val.stat = ERR_TOO_MUCH_HIT;
 		break;
 	    }
+            if (tmp.num > 0) {
+                j++;
+                if (j > maxmatch) {
+                    nmz_free_hlist(val);
+                    val.stat = ERR_TOO_MUCH_MATCH;
+                    break;
+                }
+                val = nmz_ormerge(val, tmp);
+                if (val.stat == ERR_FATAL)
+                    return val;
+                if (val.num > maxhit) {
+                    nmz_free_hlist(val);
+                    val.stat = ERR_TOO_MUCH_HIT;
+                    break;
+                }
+            }
 	    nmz_debug_printf("fw: %s, %d, %d\n", buf, tmp.num, val.num);
 	} else
 	    break;
@@ -280,6 +291,11 @@ do_word_search(const char *key, NmzResult val)
         val = nmz_get_hlist(v);
 	if (val.stat == ERR_FATAL)
 	    return val;
+        if (val.num > nmz_get_maxhit()) {
+            nmz_free_hlist(val);
+            val.stat = ERR_TOO_MUCH_HIT;
+            return val;
+        }
     } else {
         val.num  = 0;  /* no hit */
 	val.stat = SUCCESS; /* no hit but success */
@@ -340,7 +356,7 @@ cmp_phrase_hash(int hash_key, NmzResult val,
         return val;
     }
     ptr = nmz_getidxptr(phrase_index, hash_key);
-    if (ptr <= 0) {
+    if (ptr < 0) {
 	nmz_free_hlist(val);
         val.num = 0;
         return val;
@@ -535,15 +551,20 @@ do_regex_preprocessing(char *expr)
     if (*expr == '*' && expr[strlen(expr) - 1] != '*') {
         /* If suffix match such as '*bar', enforce it into regex */
         strcpy(expr, expr + 1);
-        strcat(expr, "$");
+        escape_meta_characters(expr, BUFSIZE * 2);
+        strncat(expr, "$", BUFSIZE * 2 - strlen(expr) - 1);
+        expr[BUFSIZE * 2 - 1] = '\0';
     } else if (*expr != '*' && expr[strlen(expr) - 1] == '*') {
         /* If prefix match such as 'bar*', enforce it into regex */
-        expr[strlen(expr) - 1] = '.';
-        strcat(expr, "*");
+        expr[strlen(expr) - 1] = '\0';
+        escape_meta_characters(expr, BUFSIZE * 2);
+        strncat(expr, ".*", BUFSIZE * 2 - strlen(expr) - 1);
+        expr[BUFSIZE * 2 - 1] = '\0';
     } else if (*expr == '*' && expr[strlen(expr) - 1] == '*') {
         /* If internal match such as '*foo*', enforce it into regex */
         strcpy(expr, expr + 1);
         expr[strlen(expr) - 1] = '\0';
+        escape_meta_characters(expr, BUFSIZE * 2);
     } else if (*expr == '/' && expr[strlen(expr) - 1] == '/') {
         /* Genuine regex */
         /* Remove the both of '/' chars at begging and end of string */
@@ -551,8 +572,7 @@ do_regex_preprocessing(char *expr)
         expr[strlen(expr) - 1]= '\0';
         return;
     } else {
-        char buf[BUFSIZE * 2], *bufp, *exprp;
-
+        /* field search */
         if ((*expr == '"' && expr[strlen(expr) - 1] == '"')
             || (*expr == '{' && expr[strlen(expr) - 1] == '}')) 
 	{
@@ -560,23 +580,33 @@ do_regex_preprocessing(char *expr)
             strcpy(expr, expr + 1); 
             expr[strlen(expr) - 1] = '\0';
         }
-        bufp = buf;
-        exprp = expr;
-        /* Escape meta characters */
-        while (*exprp) {
-            if (!isalnum(*exprp) && !nmz_iseuc(*exprp)) {
-                *bufp = '\\';
-                bufp++;
-            }
-            *bufp = *exprp;
-            bufp++;
-            exprp++;
-        }
-        *bufp = '\0';
-        strcpy(expr, buf);
+        escape_meta_characters(expr, BUFSIZE * 2);
     }
 }
 
+static void
+escape_meta_characters(char *expr, size_t bufsize)
+{
+    char buf[BUFSIZE * 4] = "", *bufp, *exprp;
+
+    bufp = buf;
+    exprp = expr;
+    /* Escape meta characters */
+    while (*exprp) {
+        /* japanese only */
+        if (!isalnum((unsigned char)*exprp) && !nmz_iseuc(*exprp)) {
+            *bufp = '\\';
+            bufp++;
+        }
+        *bufp = *exprp;
+        bufp++;
+        exprp++;
+    }
+    *bufp = '\0';
+    strncpy(expr, buf, bufsize - 1);
+    expr[bufsize - 1] = '\0';
+}
+
 static NmzResult 
 do_regex_search(const char *expr, NmzResult val)
 {
@@ -1089,9 +1119,9 @@ nmz_do_searching(const char *key, NmzResult src)
 
     strncpy(tmpkey, key, BUFSIZE - 1);
 
-    nmz_debug_printf("before nmz_strlower: [%s]", tmpkey);
-    nmz_strlower(tmpkey);
-    nmz_debug_printf("after nmz_strlower:  [%s]", tmpkey);
+    nmz_debug_printf("before nmz_regex_strlower: [%s]", tmpkey);
+    nmz_regex_strlower(tmpkey);
+    nmz_debug_printf("after nmz_regex_strlower:  [%s]", tmpkey);
 
     mode = detect_search_mode(tmpkey);
     if (mode == ERROR_MODE) {
@@ -1108,7 +1138,11 @@ nmz_do_searching(const char *key, NmzResult src)
 		return val;
 	    }
 	    /* Re-examine because tmpkey is wakatied. */
-	    mode = detect_search_mode(tmpkey);
+            if (strchr(tmpkey, '\t')) {
+                mode = PHRASE_MODE;
+            } else {
+                mode = WORD_MODE;
+            }
 	}
     }
 
@@ -1155,3 +1189,25 @@ nmz_free_hitnums(struct nmz_hitnumlist *hn)
     }
 }
 
+static void nmz_regex_strlower(char *str)
+{
+    if (strlen(str) >= 2 && *str == '/' && str[strlen(str) - 1] == '/') {
+        if (nmz_is_regex_searchmode()) {
+            /* keep \W \S \D \A \Z \B \G */
+            char bak = '\0';
+
+            while (*str) {
+                if (bak != '\\') {
+                    /* Using ascii dependent lower same as mknmz.  */
+                    bak = *str = _nmz_tolower(*str);
+                } else {
+                    bak = '\0';
+                }
+                str++;
+            }
+            return;
+        }
+    }
+
+    nmz_strlower(str);
+}
