@@ -1,6 +1,6 @@
 #
 # -*- Perl -*-
-# $Id: gfilter.pl,v 1.1.2.1 2001/09/21 08:11:07 knok Exp $
+# $Id: gfilter.pl,v 1.1.2.5 2004/11/01 12:56:44 opengl2772 Exp $
 # Copyright (C) 1999 Satoru Takabayashi ,
 #     This is free software with ABSOLUTELY NO WARRANTY.
 #
@@ -23,6 +23,7 @@
 #
 
 package gfilter;
+use English;
 
 # Show debug information for filters
 sub show_filter_debug_info ($$$$) {
@@ -37,6 +38,7 @@ sub show_filter_debug_info ($$$$) {
 # Adjust white spaces
 sub white_space_adjust_filter ($) {
     my ($text) = @_;
+    return undef unless defined($$text);
 
     $$text =~ s/[ \t]+/ /g;
     $$text =~ s/\r\n/\n/g;
@@ -54,18 +56,20 @@ sub filename_to_title ($$) {
     my ($cfile, $weighted_str) = @_;
 
     # for MSWin32's filename using Shift_JIS [1998-09-24]
-    if (($mknmz::SYSTEM eq "MSWin32") || ($mknmz::SYSTEM eq "os2")) {
+    if (($English::OSNAME eq "MSWin32") || ($English::OSNAME eq "os2")) {
 	$cfile = codeconv::shiftjis_to_eucjp($cfile);
-	$cfile = codeconv::eucjp_han2zen_kana($cfile);
+	codeconv::eucjp_han2zen_kana(\$cfile);
     }
-    
+
+    codeconv::normalize_eucjp(\$cfile);
+
     $cfile =~ m!^.*/([^/]*)$!;
     my $filename = $1;
 
     # get keywords from a file name.
     # modified [1998-09-18] 
     my $tmp = $filename;
-    $tmp =~ s|/\\_\.-| |g;
+    $tmp =~ tr|/\\_\.-| |;
 
     my $weight = $conf::Weight{'html'}->{'title'};
     $$weighted_str .= "\x7f$weight\x7f$tmp\x7f/$weight\x7f\n";
