@@ -1,7 +1,7 @@
 # -*- Perl -*-
-# $Id: var.pl.in,v 1.12 2000/03/02 02:39:33 satoru Exp $
+# $Id: var.pl.in,v 1.12.8.3 2006/01/24 12:42:55 opengl2772 Exp $
 # Copyright (C) 1997-1999 Satoru Takabayashi All rights reserved.
-# Copyright (C) 2000 Namazu Project All rights reserved.
+# Copyright (C) 2000-2006 Namazu Project All rights reserved.
 #     This is free software with ABSOLUTELY NO WARRANTY.
 #
 #  This program is free software; you can redistribute it and/or modify
@@ -29,9 +29,10 @@ package var;
 # Software information.
 #
 
-$VERSION         = "2.0.10";
-$COPYRIGHT       = "Copyright (C) 1997-1999 Satoru Takabayashi All rights reserved.\nCopyright (C) 2000,2001 Namazu Project All rights reserved."; # should be doublequote
+$VERSION         = "2.0.15";
+$COPYRIGHT       = "Copyright (C) 1997-1999 Satoru Takabayashi All rights reserved.\nCopyright (C) 2000-2006 Namazu Project All rights reserved."; # should be doublequote
 $MAILING_ADDRESS = 'bug-namazu@namazu.org';
+$TRAC_URI        = 'http://www.namazu.org/trac-namazu/trac.cgi';
 
 #-----------------------------------------------------------
 #
@@ -106,6 +107,10 @@ $Opt{'noencodeuri'}   = 0;
 $Opt{'nodelete'} = 0;
 $Opt{'noupdate'} = 0;
 
+$Opt{'checkfilesize'} = 0;
+
+$Opt{'decodebase64'} = 0;
+
 #-----------------------------------------------------------
 #
 # Size of `int'
