@@ -1,5 +1,5 @@
 /*
- * $Id: output.c,v 1.80.4.17 2004/03/20 15:02:51 usu Exp $
+ * $Id: output.c,v 1.80.4.24 2005/10/13 17:53:15 opengl2772 Exp $
  * 
  * Copyright (C) 1997-1999 Satoru Takabayashi All rights reserved.
  * Copyright (C) 2000-2004 Namazu Project All rights reserved.
@@ -181,10 +181,14 @@ unhtml_buffer(char *ostr) {
 	    }
 	}
 
-	if (strncasecmp(str, "<br>", 4) == 0&& *(str + 4) != '\n') {
+	if (strncasecmp(str, "<br>", 4) == 0 && *(str + 4) != '\n') {
 	    buf[i++] = '\n';
 	    str += 3;
 	    continue;
+	} else if (strncasecmp(str, "<br />", 6) == 0 && *(str + 6) != '\n') {
+	    buf[i++] = '\n';
+	    str += 5;
+	    continue;
 	}
 	if (*str == '<') {
 	    f = 1;
@@ -243,7 +247,7 @@ print_hitnum_each (struct nmz_hitnumlist *hn)
 	!is_listmode() && !is_quietmode()) 
     {
 	do {
-	    char *converted = nmz_codeconv_external(hnptr->word);
+	    char *converted = nmz_query_external(hnptr->word);
 	    if (converted == NULL) {
 		die("print_hitnum_each");
 	    }
@@ -490,7 +494,7 @@ print_query(const char * qs, int w)
 	if (strncmp(qs, "whence=", strlen("whence=")) == 0) {
 	    foo = 1;
 	    printf("whence=%d", w);
-	    for (qs += strlen("whence="); isdigit(*qs); qs++);
+	    for (qs += strlen("whence="); isdigit((unsigned char)*qs); qs++);
 	} else {
 	    /* '"' is converted to entities "&quot;" */
 	    putc_entitize(*qs);
@@ -547,7 +551,7 @@ print_page_index(int n)
 	printf("[%d]", i + 1);
 	if (is_htmlmode()) {
 	    if (i * max != whence) {
-		printf("</A> ");
+		printf("</a> ");
 	    } else
 		printf("</strong> ");
 	}
@@ -623,6 +627,11 @@ print_hitnum_all_idx(void)
 	    if (nmz_get_idxnum() > 1 && nmz_get_querytokennum() > 1) {
 	        printf(_(" [ TOTAL: %d ]"), nmz_get_idx_totalhitnum(idxid));
 	    }
+            if (nmz_get_idxnum() > 1) {
+                if (is_htmlmode()) {
+                   printf("</li>");
+                }
+            }
 	    printf("\n");
 	}
     }
@@ -676,14 +685,15 @@ print_msgfile(const char *fname) {
 static void 
 print_range(NmzResult hlist)
 {
-    if (is_htmlmode())
-        printf("<p>\n");
+    if (is_htmlmode()) {
+        printf("<div class=\"namazu-result-footer\">\n");
+    }
     print_current_range(hlist.num);
     if (is_pageindex()) {
         print_page_index(hlist.num);
     }
     if (is_htmlmode()) {
-        printf("</p>\n");
+        printf("</div>\n");  /* class="namazu-result-footer" */
     } else {
         printf("\n");
     }
@@ -723,6 +733,12 @@ print_result(NmzResult hlist, const char *query, const char *subquery)
 	return FAILURE;
     }
 
+    if (!is_countmode() && !is_listmode() && !is_quietmode()) {
+        if (is_htmlmode()) {
+            fputs("<div class=\"namazu-result-header\">\n", stdout);
+        }
+    }
+
     /* Result1:  <h2>Results:</h2>, References:  */
     if (is_refprint() && !is_countmode() && 
 	!is_listmode() && !is_quietmode()) 
@@ -763,6 +779,9 @@ print_result(NmzResult hlist, const char *query, const char *subquery)
     if (hlist.num > 0) {
         if (!is_countmode() && !is_listmode() && !is_quietmode()) {
             print_hitnum(hlist.num);  /* <!-- HIT -->%d<!-- HIT --> */
+            if (is_htmlmode()) {
+                fputs("</div>\n", stdout); /* class="namazu-result-header" */
+            }
         }
 	if (is_countmode()) {
 	    printf("%d\n", hlist.num);
@@ -783,6 +802,7 @@ print_result(NmzResult hlist, const char *query, const char *subquery)
         } else if (!is_listmode() && !is_quietmode()) {
 	    html_print(_("	<p>No document matching your query.</p>\n"));
 	    if (is_htmlmode()) {
+                fputs("</div>\n", stdout); /* class="namazu-result-header" */
 		print_msgfile(NMZ.tips);
 	    }
         }
@@ -801,7 +821,9 @@ print_result(NmzResult hlist, const char *query, const char *subquery)
 void 
 print_default_page (void) {
     if (is_htmlmode()) {
-	printf("%s %s" CRLF CRLF, MSG_MIME_HEADER, contenttype);
+        if (is_cgimode()) {
+	    printf("%s %s" CRLF CRLF, MSG_MIME_HEADER, contenttype);
+        }
 	print_headfoot(NMZ.head, "", "");
 	print_msgfile(NMZ.body);
 	print_headfoot(NMZ.foot, "", "");
